// Copyright (C) 2010 - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function X = sudoku_latinsquare ( varargin )
  //   Create Latin Squares of order n.
  //
  // Calling Sequence
  //   X = sudoku_latinsquare ( n )
  //   X = sudoku_latinsquare ( n , randomized )
  //
  // Parameters
  // n: a positive integer 1,2, ...
  // X: a n x n matrix with integers from 1, 2, ..., n
  // randomized: a boolean. If true, rows and columns are permuted. (default = %t)
  //
  //
  // Description
  //   Generates a Latin Square of order n. In each row 
  //   or column of the matrix, all integers from 1 to n 
  //   are present only once.
  //
  // Examples
  // X = sudoku_latinsquare ( 4 )
  // X = sudoku_latinsquare ( 4 , %f )
  //
  // Authors
  //   Michael Baudin, 2010

  [lhs,rhs]=argn();
  if ( rhs < 1 | rhs > 2 ) then
    errmsg = msprintf(gettext("%s: Unexpected number of input arguments : %d provided while 1 or 2 are expected."), "sudoku_latinsquare", rhs);
    error(errmsg)
  end
  n = varargin(1)
  if ( rhs < 2 ) then
    randomized = %t
  else
    randomized = varargin(2)
  end

  X = zeros(n,n)
  for i = 1 : n
    X(i,:) = modulo((1:n)+(i-2),n) + 1
  end
  if ( randomized ) then
    r = grand(1,"prm",(1:n)')'
    X(:,r) = X(:,:)
    r = grand(1,"prm",(1:n)')
    X(r,:) = X(:,:)
  end
endfunction

