// Copyright (C) 2010 - DIGITEO - Michael Baudin
//
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

// <-- JVM NOT MANDATORY -->
// <-- ENGLISH IMPOSED -->


//
// assert_close --
//   Returns 1 if the two real matrices computed and expected are close,
//   i.e. if the relative distance between computed and expected is lesser than epsilon.
// Arguments
//   computed, expected : the two matrices to compare
//   epsilon : a small number
//
function flag = assert_close ( computed, expected, epsilon )
  if expected==0.0 then
    shift = norm(computed-expected);
  else
    shift = norm(computed-expected)/norm(expected);
  end
  if shift < epsilon then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction
//
// assert_equal --
//   Returns 1 if the two real matrices computed and expected are equal.
// Arguments
//   computed, expected : the two matrices to compare
//   epsilon : a small number
//
function flag = assert_equal ( computed , expected )
  if computed==expected then
    flag = 1;
  else
    flag = 0;
  end
  if flag <> 1 then pause,end
endfunction


str = [
"6.......7"
"....9..2."
"3.1..259."
"8....7.13"
"....8...."
"76.3....8"
".782..1.6"
".5..3...."
"2.......9"
];
[fd,err]=mopen("minimal.sdk", "w")
for i = 1 : size(str,"r")
  fprintf(fd,str(i)+"\n")
end
err=mclose(fd)

X = sudoku_readsdk ( "minimal.sdk" );
XE = [
6 0 0   0 0 0   0 0 7 
0 0 0   0 9 0   0 2 0 
3 0 1   0 0 2   5 9 0 
..
8 0 0   0 0 7   0 1 3 
0 0 0   0 8 0   0 0 0 
7 6 0   3 0 0   0 0 8 
..
0 7 8   2 0 0   1 0 6 
0 5 0   0 3 0   0 0 0 
2 0 0   0 0 0   0 0 9 
];
assert_equal ( X , XE );

str = [
"#A SadMan Software"
"#D Straightforward enough"
"#C watch for the naked pair"
"#B 27/08/2009"
"#S SadMan Software Sudoku"
"#L Mild"
"#U http://www.sadmansoftware.com/sudoku/"
"#N 1"
"#H 27"
"6.......7"
"....9..2."
"3.1..259."
"8....7.13"
"....8...."
"76.3....8"
".782..1.6"
".5..3...."
"2.......9"
]
[fd,err]=mopen("minimal2.sdk", "w")
for i = 1 : size(str,"r")
  fprintf(fd,str(i)+"\n")
end
err=mclose(fd)
X = sudoku_readsdk ("minimal2.sdk");
XE = [
6 0 0   0 0 0   0 0 7 
0 0 0   0 9 0   0 2 0 
3 0 1   0 0 2   5 9 0 
..
8 0 0   0 0 7   0 1 3 
0 0 0   0 8 0   0 0 0 
7 6 0   3 0 0   0 0 8 
..
0 7 8   2 0 0   1 0 6 
0 5 0   0 3 0   0 0 0 
2 0 0   0 0 0   0 0 9 
];
assert_equal ( X , XE );

